#ifndef CONJGRAD
#define CONJGRAD
#include <iostream>

/**************************************************************
This is a template based linear equation (minimization) solver
using the conjugate gradient method.

Programmed by Luis Silvestre Feb-2007
***************************************************************
The following methods are required by the template:
V& V::operator=(const V&)
double V::norm2()
double V::operator*(const V&)
int V::size()
V operator*(double a, const V&)
V M::operator()(const V&)
***************************************************************/

#define EPSILON (1e-11)

template<class M, class V>
V conjugateGradients(const M& matrix, const V& initialGuess, const V& b){
  int n= b.size();
  V x(initialGuess);
  V d(b - matrix(x));
  V r(d);
  double r2 = r.norm2();
  for(int i=0; (i<n)&&(r2>EPSILON)  ; ++i){
    if (i%25==1) cout << "it: " << i << " -- err^2: " << r2/n << endl;
    V q(matrix(d));
    double a = r2 / (d * q);
    x += a * d;
    if(i%50!=0) r -= a * q;
      else r = b - matrix(x);
    double r22 = r.norm2();
    double b = r22 / r2;
    r2 = r22;
    d = r + b * d;
  }
  return x;
}

#endif