#include "grid3d.h"
#include <cstring>
#include <cmath>

using namespace std;

grid3d::grid3d(int dim){
  n = dim;
  m.resize(n*n*n);
}

grid3d::grid3d(const grid3d& g){
  n = g.n;
  m = g.m;
}

grid3d::~grid3d(){
}

double & grid3d::at(int i, int j, int k){
  return m[i+n*j+n*n*k];
}

double & grid3d::operator() (int i, int j, int k){
  return at(i,j,k);
}

double grid3d::get(int i, int j, int k) const{
  return m[i+n*j+n*n*k];
}

grid3d& grid3d::operator=(const grid3d & g){
  n = g.dimension();
  m = g.m;
  return (*this);
}

grid3d& grid3d::operator+=(const grid3d & g){
  int k = m.size();
  for(int i=0; i<k; ++i) m[i] += g.m[i];
  return (*this);
}

grid3d& grid3d::operator-=(const grid3d & g){
  int k = m.size();
  for(int i=0; i<k; ++i) m[i] -= g.m[i];
  return (*this);
}

void grid3d::clear(){
  int n3 = n*n*n;
  for(int i=0; i<n3; ++i) m[i]=0;
}

grid3d grid3d::operator+(const grid3d & g) const{
  grid3d res(n);
  int n3 = size();
  for(int i=0; i<n3; ++i){
    res.m[i] = m[i] + g.m[i];
  }
  return res;
}

grid3d grid3d::operator-(const grid3d & g) const{
  grid3d res(n);
  int n3 = size();
  for(int i=0; i<n3; ++i){
    res.m[i] = m[i] - g.m[i];
  }
  return res;
}

double grid3d::norm() const{
  double a=0;
  int n3 = size();
  for(int i=0; i<n3; ++i){
    a >?= abs(m[i]);
  }
  return a;
}

inline double sqr(double x){return x*x;}

double grid3d::norm2() const{
  double a=0;
  int n3 = size();
  for(int i=0; i<n3; ++i){
    a += sqr(m[i]);
  }
  return a;
}

double grid3d::operator*(const grid3d & g) const{
  double r=0;
  int n3 = size();
  for(int i=0; i<n3; ++i){
    r += m[i]*g.m[i];
  }
  return r;
}

grid3d operator*(double a, const grid3d &g){
  grid3d r(g.n);
  int n=g.size();
  for(int i=0; i<n; ++i){
    r.m[i] = a*g.m[i];
  }
  return r;
}

grid3d grid3d::interpolate(int m){
  grid3d r(m);
  for(int x=0; x<m; ++x){
    for(int y=0; y<m; ++y){
      for(int z=0; z<m; ++z){
        int x0 = int((double(x) / (m-1)) * (n-1));
        int y0 = int((double(y) / (m-1)) * (n-1));
        int z0 = int((double(z) / (m-1)) * (n-1));
        double cx = ((double(x0+1) / (n-1)) - (double(x) / (m-1))) * (n-1);
        double cy = ((double(y0+1) / (n-1)) - (double(y) / (m-1))) * (n-1);
        double cz = ((double(z0+1) / (n-1)) - (double(z) / (m-1))) * (n-1);
        int x01 = (x0==n-1) ? n-1 : x0+1;
        int y01 = (y0==n-1) ? n-1 : y0+1;
        int z01 = (z0==n-1) ? n-1 : z0+1;
        r(x,y,z) = cx*cy*cz*get(x0,y0,z0) + (1-cx)*cy*cz*get(x01,y0,z0)
            + cx*(1-cy)*cz*get(x0,y01,z0)+ (1-cx)*(1-cy)*cz*get(x01,y01,z0)
            + cx*cy*(1-cz)*get(x0,y0,z01) + (1-cx)*cy*(1-cz)*get(x01,y0,z01) 
            + cx*(1-cy)*(1-cz)*get(x0,y01,z01) + (1-cx)*(1-cy)*(1-cz)*get(x01,y01,z01);
      }
    }
  }
  return r;
}
